unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, hyieutils, iexBitmaps, hyiedefs,
  iesettings, iexLayers, iexRulers, iexToolbars, iexUserInteractions,
  Vcl.ExtCtrls, ieview, imageenview, iexColorButton, Vcl.StdCtrls,
  Vcl.ComCtrls;

type
  TMyTrackBar = class(TTrackBar)
    public
      property OnMouseUp;
  end;
  TTrackBar = class(TMyTrackBar);

type
  TForm1 = class(TForm)
    ImageEnView1: TImageEnView;
    pnlReplaceColor: TPanel;
    pnlColors: TPanel;
    pnlClickedColor: TPanel;
    btnTargetColor: TIEColorButton;
    btnLoadImage: TButton;
    btnUndo: TButton;
    pnlToleranceHue: TPanel;
    lblToleranceHue: TLabel;
    trackbarToleranceHue: TTrackBar;
    pnlColorReplaceControls: TPanel;
    pnlToleranceSat: TPanel;
    lblToleranceSat: TLabel;
    trackbarToleranceSat: TTrackBar;
    pnlToleranceLum: TPanel;
    lblToleranceLum: TLabel;
    trackbarToleranceLum: TTrackBar;
    btnResetSliders: TButton;
    procedure MyTrackBarMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FormCreate(Sender: TObject);
    procedure btnLoadImageClick(Sender: TObject);
    procedure btnResetSlidersClick(Sender: TObject);
    procedure btnTargetColorChange(Sender: TObject);
    procedure btnUndoClick(Sender: TObject);
    procedure ImageEnView1MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure ImageEnView1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure trackbarToleranceHueChange(Sender: TObject);
    procedure trackbarToleranceHueKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure trackbarToleranceKeyUp(Sender: TObject; var Key: Word; Shift:
        TShiftState);
    procedure trackbarToleranceLumChange(Sender: TObject);
    procedure trackbarToleranceSatChange(Sender: TObject);
  private
    { Private declarations }
    FlblToleranceHueCaption: string;
    FlblToleranceSatCaption: string;
    FlblToleranceLumCaption: string;
    FLastClickedColor: TColor;

    procedure DoReplaceLastClickedColor;
    function ScreenPosToColor(X, Y: Integer) : TColor;
    procedure ReplaceColor(aColor: TColor = clNone);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses
  CodeSiteLogging;

procedure TForm1.MyTrackBarMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  DoReplaceLastClickedColor;
end;

procedure TForm1.DoReplaceLastClickedColor;
begin
  if FLastClickedColor <> clNone then
    ReplaceColor(FLastClickedColor);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  ImageEnView1.Proc.UndoLimit := 20;

  FlblToleranceHueCaption := lblToleranceHue.Caption;
  FlblToleranceSatCaption := lblToleranceSat.Caption;
  FlblToleranceLumCaption := lblToleranceLum.Caption;

  trackbarToleranceHueChange(nil);
  trackbarToleranceSatChange(nil);
  trackbarToleranceLumChange(nil);

  FLastClickedColor := clNone;

  trackbarToleranceHue.OnMouseUp := MyTrackBarMouseUp;
  trackbarToleranceSat.OnMouseUp := MyTrackBarMouseUp;
  trackbarToleranceLum.OnMouseUp := MyTrackBarMouseUp;
end;

procedure TForm1.ReplaceColor(aColor: TColor = clNone);
var
  H, S, V: Integer;
  ThisColor: TColor;
begin
  if ImageEnView1.IsEmpty2 then EXIT;

  if aColor = clNone then // default call by clicking in the image
    ThisColor := pnlClickedColor.Color
  else
    ThisColor := aColor; // when called by TrackBarMouseUp

  ColorToHSV(ThisColor, H, S, V); // this is the color picked from the image by clicking in the image

  ImageEnView1.Proc.CastColorRange(
    HSVToColor(H - trackbarToleranceHue.Position, S + trackbarToleranceSat.Position, V - trackbarToleranceLum.Position), // BeginColor
    HSVToColor(H + trackbarToleranceHue.Position, S - trackbarToleranceSat.Position, V + trackbarToleranceLum.Position), // EndColor
    btnTargetColor.SelectedColor); // CastColor

  ImageEnView1.Update;

  FLastClickedColor := ThisColor;
end;

function TForm1.ScreenPosToColor(X, Y: Integer) : TColor;
var
  bmpX, bmpY: Integer;
begin
  bmpX := ImageEnView1.XScr2Bmp( X );
  bmpY := ImageEnView1.YScr2Bmp( Y );

  if ( bmpX < 0 ) or ( bmpX >= ImageEnView1.IEBitmap.width ) or
     ( bmpY < 0 ) or ( bmpY >= ImageEnView1.IEBitmap.Height ) then
    Result := clNone // ImageEnView1.Background // Invalid bitmap color
  else
    Result := TRGB2TColor( ImageEnView1.IEBitmap.Pixels[ bmpX, bmpY ]);
end;

procedure TForm1.btnLoadImageClick(Sender: TObject);
var
  sFilename: string;
begin
  sFilename := ImageEnView1.IO.ExecuteOpenDialog;
  if sFilename <> '' then
  begin
    ImageEnView1.IO.LoadFromFile(sFileName);
    pnlColorReplaceControls.Enabled := True;
  end;
end;

procedure TForm1.btnResetSlidersClick(Sender: TObject);
begin
  trackbarToleranceHue.Position := 0;
  trackbarToleranceSat.Position := 0;
  trackbarToleranceLum.Position := 0;
end;

procedure TForm1.btnTargetColorChange(Sender: TObject);
begin
  //
end;

procedure TForm1.btnUndoClick(Sender: TObject);
begin
  ImageEnView1.Proc.Undo( True );
end;

procedure TForm1.ImageEnView1MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if ImageEnView1.IsEmpty2 then EXIT;

  pnlClickedColor.Color := ScreenPosToColor( X, Y );

  if pnlClickedColor.Color <> clNone then
    ReplaceColor;
end;

procedure TForm1.ImageEnView1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  if ImageEnView1.IsEmpty2 then EXIT;

  pnlClickedColor.Color := ScreenPosToColor( X, Y );
end;

procedure TForm1.trackbarToleranceHueChange(Sender: TObject);
begin
  lblToleranceHue.Caption := FlblToleranceHueCaption + ' (' + IntToStr(trackbarToleranceHue.Position) + ')';
end;

procedure TForm1.trackbarToleranceHueKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  case Key of
    48, VK_NUMPAD0: trackbarToleranceHue.Position := 0;
  end;
end;

procedure TForm1.trackbarToleranceKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  DoReplaceLastClickedColor;
end;

procedure TForm1.trackbarToleranceLumChange(Sender: TObject);
begin
  lblToleranceLum.Caption := FlblToleranceLumCaption + ' (' + IntToStr(trackbarToleranceLum.Position) + ')';
end;

procedure TForm1.trackbarToleranceSatChange(Sender: TObject);
begin
  lblToleranceSat.Caption := FlblToleranceSatCaption + ' (' + IntToStr(trackbarToleranceSat.Position) + ')';
end;

end.
